/*
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {CookieHandler} from './cookie-handler';
import {Globals} from './domain/globals';
import {Layout} from './domain/layout';
import {Options} from './widget/options';
import {Theme} from './domain/theme';
import {Uhr} from './uhr';

declare var $: JQueryStatic;

export class UhrRenderer {
    private cookieHandler: CookieHandler;

    public constructor(
        private readonly uhr: Uhr,
        private readonly $element: JQuery<HTMLElement>,
        private readonly options: Options,
        private readonly id: string
    ) {
        this.cookieHandler = new CookieHandler(id, options.cookiePath);
    }

    public render(): void {
        this.setupHTML();
        this.wireFunctionality();
    }

    public setWidth(width: string): void {
        this.$element.css('width', width);
        const realWidth: number = this.$element.width();
        this.$element.width(realWidth);
        this.$element.height(realWidth);
        this.$element.css('font-size', (realWidth / 40) + 'px');
    }

    private setupHTML(): void {
        // Base clock area
        this.$element.addClass('uhr')
            .empty()
            .append('<span class="item dot dot1"></span>')
            .append('<span class="item dot dot2"></span>')
            .append('<span class="item dot dot3"></span>')
            .append('<span class="item dot dot4"></span>')
            .append('<div class="letterarea"></div>')
            .append('<div class="reflection"></div>');

        this.setWidth(this.options.width);

        if (this.options.controls) {
            const controlpanel = $(`<div class="uhr-controlpanel" id="uhr-controlpanel${this.id}"></div>`);
            const content = $('<div class="content"></div>');
            controlpanel.append(content);
            // on/off switch
            const toggleSwitch = $(`<div class="onoffswitch" id="uhr-onoffswitch${this.id}"></div>`);
            toggleSwitch.append(`<input type="checkbox" class="onoffswitch-checkbox" id="uhr-onoffswitch-checkbox${this.id}" checked="checked" />`);
            toggleSwitch.append(`<label class="onoffswitch-label" for="uhr-onoffswitch-checkbox${this.id}"><div class="onoffswitch-inner"></div><div class="onoffswitch-switch"></div></label>`);
            content.append(toggleSwitch);

            // time mode switch
            const modeSwitch = $(`<div class="onoffswitch" id="uhr-modeswitch${this.id}"></div>`);
            modeSwitch.append(`<input type="checkbox" class="onoffswitch-checkbox" id="uhr-modeswitch-checkbox${this.id}" checked="checked" />`);
            modeSwitch.append(`<label class="onoffswitch-label" for="uhr-modeswitch-checkbox${this.id}"><div class="modeswitch-inner"></div><div class="onoffswitch-switch"></div></label>`);
            content.append(modeSwitch);
            // language chooser
            if (Globals.hasMultipleLayouts()) {
                const languageChooser = $(`<select id="uhr-languagechooser${this.id}"></select>`);
                Globals.getLayouts().forEach((layout: Layout): void => {
                    languageChooser.append(`<option value="${layout.code}">${layout.prettyName}</option>`);
                });
                content.append(languageChooser);
            }

            // theme chooser
            if (Globals.hasMultipleThemes()) {
                const themeChooser = $(`<select id="uhr-themechooser${this.id}"></select>`);
                Globals.getThemes().forEach((theme: Theme): void => {
                    themeChooser.append(`<option value="${theme.styleClass}">${theme.name}</option>`);
                });
                content.append(themeChooser);
            }
            const closebutton: JQuery<HTMLElement> = $(`<a class="uhr-closecontrolpanel" id="uhr-closecontrolpanel${this.id}"></a>`);
            closebutton.on({click: (): JQuery<HTMLElement> => $(`#uhr-controlpanel${this.id}`).hide('fast')});
            content.append(closebutton);
            this.$element.after(controlpanel);
            controlpanel.hide();
            const configlink: JQuery<HTMLElement> = $(`<a class="uhr-configlink" id="uhr-configlink${this.id}"></a>`);
            configlink.on({click: (): void => this.toggleConfigScreen()});
            this.$element.after(configlink);
        }
    }

    private wireFunctionality(): void {
        // on/off switch
        const toggleSwitch: JQuery<HTMLElement> = $(`#uhr-onoffswitch-checkbox${this.id}`);
        toggleSwitch.on({click: (): void => this.uhr.toggle()});
        let status = this.cookieHandler.getStatus();
        if (status === undefined || this.options.force) {
            status = this.options.status;
        }
        toggleSwitch.prop('checked', status === 'on');
        if (status === 'on') {
            this.uhr.start();
        } else {
            this.uhr.stop();
        }

        // time mode switch
        const modeSwitch: JQuery<HTMLElement> = $(`#uhr-modeswitch-checkbox${this.id}`);
        modeSwitch.on({
            click: (): void => {
                if (this.options.mode === 'seconds') {
                    this.uhr.setMode('normal');
                } else {
                    this.uhr.setMode('seconds');
                }
            }
        });

        let mode = this.cookieHandler.getMode();
        if (mode === undefined || this.options.force) {
            mode = this.options.mode;
        }
        modeSwitch.prop('checked', mode !== 'seconds');
        if (mode === 'seconds') {
            this.uhr.setMode('seconds');
        } else {
            this.uhr.setMode('normal');
        }

        // language chooser
        const languageChooser: JQuery<HTMLElement> = $(`#uhr-languagechooser${this.id}`);
        languageChooser.on({
            change: (): void => {
                const languageKey = $(`#uhr-languagechooser${this.id}`).val() as string;
                this.uhr.setLayout(languageKey);
            }
        });
        let selectedLayout = this.cookieHandler.getLayout();
        if (selectedLayout === undefined || this.options.force) {
            selectedLayout = this.options.language;
        }
        let found = Globals.getLayouts().some((item: Layout): boolean => selectedLayout === item.code);
        if (!found) {
            let fallbackLanguage;
            if (Globals.hasLayouts()) {
                fallbackLanguage = Globals.getFirstLayout().code;
            } else {
                fallbackLanguage = '';
            }
            console.warn(`Language '${selectedLayout}' not found! Using fallback '${fallbackLanguage}'.`);
            selectedLayout = fallbackLanguage;
        }
        languageChooser.val(selectedLayout);
        this.options.language = '';
        this.uhr.setLayout(selectedLayout);

        // theme chooser
        const themeChooser: JQuery<HTMLElement> = $(`#uhr-themechooser${this.id}`);
        themeChooser.on({
            change: (): void => {
                const themeKey = $(`#uhr-themechooser${this.id}`).val() as string;
                this.uhr.setTheme(themeKey);
            }
        });
        let selectedTheme = this.cookieHandler.getTheme();
        if (selectedTheme === undefined || this.options.force) {
            selectedTheme = this.options.theme;
        }
        found = Globals.getThemes().some((item: Theme): boolean => selectedTheme === item.styleClass);
        if (!found) {
            const fallbackTheme = Globals.getFirstTheme().styleClass;
            console.warn(`Theme '${selectedTheme}' not found! Using fallback '${fallbackTheme}'.`);
            selectedTheme = fallbackTheme;
        }
        themeChooser.val(selectedTheme);
        this.options.theme = '';
        this.uhr.setTheme(selectedTheme);
        if (this.options.autoresize) {
            $(window).on({
                resize: (): void => {
                    const $parent: JQuery<HTMLElement> = this.$element.parent();
                    const $window: JQuery<Window> = $(window);
                    const parentWidth: number = $parent.width();
                    const parentHeight: number = $parent.height();
                    const windowWidth: number = $window.width();
                    const windowHeight: number = $window.height();
                    const size: string = `${Math.min(parentWidth, parentHeight, windowWidth, windowHeight)}px`;
                    this.setWidth(size);
                }
            });
        }
    }

    private toggleConfigScreen(): void {
        $(`#uhr-controlpanel${this.id}`).toggle('fast');
    }
}
